@@ -133,8 +133,9 @@ enum {
 #define ONE_SECOND (1000 * 1000)  /* 1 second, or 1000 milliseconds; eternity, in other words */
 
 #include "release.h"
-
+#if 0
 static char *version = "1.2 (" RELEASE_VERSION ")";
+#endif
 static char *dev_info = "ath_rate_minstrel";
 
 #define STALE_FAILURE_TIMEOUT_MS 10000
@@ -198,7 +199,7 @@ calc_usecs_unicast_packet(struct ath_softc *sc, int length,
 		unsigned int x = 0, tt = 0;
 		unsigned int cix = rt->info[rix].controlRate;
 		int rts = 0, cts = 0;
-		int cw = WIFI_CW_MIN;
+		int cw = ATH_DEFAULT_CWMIN;
 
 		KASSERT(rt != NULL, ("no rate table, mode %u", sc->sc_curmode));
 
@@ -256,7 +257,7 @@ calc_usecs_unicast_packet(struct ath_softc *sc, int length,
 
 			if (!rt->info[cix].rateKbps) {
 #if 0
-				printk(KERN_WARNING 
+				printk(KERN_WARNING
 					"cix %d (%d) bad ratekbps %d mode %u\n",
 					cix, rt->info[cix].dot11Rate,
 					rt->info[cix].rateKbps,
@@ -283,7 +284,7 @@ calc_usecs_unicast_packet(struct ath_softc *sc, int length,
 		tt += (long_retries + 1) * ath_hal_computetxtime(sc->sc_ah, rt, length,
 							rix, AH_TRUE);
 		for (x = 0; x <= short_retries + long_retries; x++) {
-			cw = MIN(WIFI_CW_MAX, (cw + 1) * 2);
+			cw = MIN(ATH_DEFAULT_CWMAX, (cw + 1) * 2);
 			tt += (t_slot * cw / 2);
 		}
 		return tt;
@@ -308,7 +309,7 @@ ath_rate_node_copy(struct ath_softc *sc,
 		struct ath_node *dst, const struct ath_node *src)
 {
 		struct minstrel_node *odst = ATH_NODE_MINSTREL(dst);
-		const struct minstrel_node *osrc = 
+		const struct minstrel_node *osrc =
 			(const struct minstrel_node *)&src[1];
 		memcpy(odst, osrc, sizeof(struct minstrel_node));
 }
@@ -331,8 +332,8 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 			    return;
 		}
 
-		mrr = sc->sc_mrretry && 
-			!(ic->ic_flags & IEEE80211_F_USEPROT) && 
+		mrr = sc->sc_mrretry &&
+			!(ic->ic_flags & IEEE80211_F_USEPROT) &&
 			ENABLE_MRR;
 
 		if (sn->static_rate_ndx >= 0) {
@@ -342,8 +343,8 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 			sn->random_n = (sn->a * sn->random_n) + sn->b;
 			offset = sn->random_n & 0xf;
 
-			if ((((100 * sn->sample_count) / sn->packet_count) < 
-					 ath_lookaround_rate) && 
+			if ((((100 * sn->sample_count) / sn->packet_count) <
+					 ath_lookaround_rate) &&
 					(offset < 2)) {
 				sn->sample_count++;
 				sn->is_sampling = 1;
@@ -372,7 +373,7 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 				}
 				sn->rs_sample_rate = ndx;
 				sn->rs_sample_rate_slower =
-					(sn->perfect_tx_time[ndx] > 
+					(sn->perfect_tx_time[ndx] >
 					 sn->perfect_tx_time[sn->max_tp_rate]);
 				if (sn->rs_sample_rate_slower)
 					ndx = sn->max_tp_rate;				
@@ -386,7 +387,7 @@ ath_rate_findrate(struct ath_softc *sc, struct ath_node *an,
 			    *try0 = sn->retry_adjusted_count[ndx];
 
 		KASSERT((ndx < sn->num_rates),
-			    ("%s: bad ndx (%d/%d) for " MAC_FMT "?\n",
+			    ("%s: bad ndx (%d/%d) for " MAC_FMT "?",
 			     dev_info, ndx, sn->num_rates,
 			     MAC_ADDR(an->an_node.ni_macaddr)));
 
@@ -406,6 +407,14 @@ ath_rate_get_mrr(struct ath_softc *sc, struct ath_node *an, int shortPreamble,
 		/* Index into the rate table, so for example, it is  0..11. */
 		int rc1, rc2, rc3;
 
+		if (sn->num_rates <= 0) {
+			DPRINTF(sc, "%s: no rates for " MAC_FMT "\n",
+					dev_info,
+					MAC_ADDR(an->an_node.ni_macaddr));
+			memset(mrr, 0, sizeof(struct ieee80211_mrr));
+			return;
+		}
+
 		if (sn->is_sampling) {
 			sn->is_sampling = 0;
 			if (sn->rs_sample_rate_slower)
@@ -420,17 +429,17 @@ ath_rate_get_mrr(struct ath_softc *sc, struct ath_node *an, int shortPreamble,
 		rc3 = 0;
 
 		KASSERT((rc1 >= 0) && (rc1 < sn->num_rates),
-			    ("%s: bad rc1 (%d/%d) for " MAC_FMT "?\n",
+			    ("%s: bad rc1 (%d/%d) for " MAC_FMT "?",
 			     dev_info, rc1, sn->num_rates,
 			     MAC_ADDR(an->an_node.ni_macaddr)));
 
 		KASSERT((rc2 >= 0) && (rc2 < sn->num_rates),
-			    ("%s: bad rc2 (%d/%d) for " MAC_FMT "?\n",
+			    ("%s: bad rc2 (%d/%d) for " MAC_FMT "?",
 			     dev_info, rc2, sn->num_rates,
 			     MAC_ADDR(an->an_node.ni_macaddr)));
 
 		KASSERT((rc3 >= 0) && (rc3 < sn->num_rates),
-			    ("%s: bad rc3 (%d/%d) for " MAC_FMT "?\n",
+			    ("%s: bad rc3 (%d/%d) for " MAC_FMT "?",
 			     dev_info, rc3, sn->num_rates,
 			     MAC_ADDR(an->an_node.ni_macaddr)));
 
@@ -483,7 +492,7 @@ ath_rate_tx_complete(struct ath_softc *sc,
 		/* 'tries' is the total number of times we have endeavoured to
 		 * send this packet, and is a sum of the #attempts at each
 		 * level in the multi-rate retry chain. */
-		tries = ts->ts_shortretry + ts->ts_longretry + 1;
+		tries = ts->ts_longretry + 1;
 
 		if (sn->num_rates <= 0) {
 			DPRINTF(sc, "%s: " MAC_FMT " %s no rates yet\n", dev_info,
@@ -494,8 +503,8 @@ ath_rate_tx_complete(struct ath_softc *sc,
 		if (!ts->ts_status)  /* Success when sending a packet*/
 			sn->rs_ratesuccess[final_ndx]++;
 
-		mrr = sc->sc_mrretry && 
-			!(ic->ic_flags & IEEE80211_F_USEPROT) && 
+		mrr = sc->sc_mrretry &&
+			!(ic->ic_flags & IEEE80211_F_USEPROT) &&
 			ENABLE_MRR;
 
 		if (!mrr) {
@@ -506,10 +515,10 @@ ath_rate_tx_complete(struct ath_softc *sc,
 			return;
 		}
 
-		/* Now, query the HAL/hardware to find out the contents of the 
-		 * multirate retry chain. If we have it set to 6, 3, 2, 2, this 
+		/* Now, query the HAL/hardware to find out the contents of the
+		 * multirate retry chain. If we have it set to 6, 3, 2, 2, this
 		 * call will always return 6,3,2,2. For some packets, we can
-		 * get a mrr of 0, -1, -1, -1, which indicates there is no 
+		 * get a mrr of 0, -1, -1, -1, which indicates there is no
 		 * chain installed for that packet */
 		rate0 = sc->sc_hwmap[MS(ds->ds_ctl3, AR_XmitRate0)].ieeerate;
 		tries0 = MS(ds->ds_ctl2, AR_XmitDataTries0);
@@ -677,9 +686,9 @@ ath_rate_ctl_reset(struct ath_softc *sc, struct ieee80211_node *ni)
 		 * the node.  We know the rate is there because the
 		 * rate set is checked when the station associates. */
 		/* NB: the rate set is assumed to be sorted. */
-		for (; 
-				(srate >= 0) && 
-				(ni->ni_rates.rs_rates[srate] & 
+		for (;
+				(srate >= 0) &&
+				(ni->ni_rates.rs_rates[srate] &
 				 IEEE80211_RATE_VAL) != vap->iv_fixed_rate;
 				srate--);
 
@@ -718,7 +727,7 @@ ath_rate_ctl_reset(struct ath_softc *sc, struct ieee80211_node *ni)
 		sn->retry_adjusted_count[x] = 1;
 
 		for (retry_index = 2; retry_index < ATH_TXMAXTRY; retry_index++) {
-			tx_time = calc_usecs_unicast_packet(sc, 1200, 
+			tx_time = calc_usecs_unicast_packet(sc, 1200,
 					sn->rates[x].rix, 0, retry_index);
 			if (tx_time > ath_segment_size)
 				break;
@@ -730,7 +739,7 @@ ath_rate_ctl_reset(struct ath_softc *sc, struct ieee80211_node *ni)
 #if 0
 	DPRINTF(sc, "%s: Retry table for this node\n", __func__);
 	for (x = 0; x < ni->ni_rates.rs_nrates; x++)
-		DPRINTF(sc, "%2d  %2d %6d  \n", x, sn->retry_count[x], 
+		DPRINTF(sc, "%2d  %2d %6d  \n", x, sn->retry_count[x],
 				sn->perfect_tx_time[x]);
 #endif
 
@@ -757,12 +766,12 @@ ath_rate_newstate(struct ieee80211vap *vap, enum ieee80211_state newstate)
 
 		if (newstate == IEEE80211_S_RUN) {
 			if (ic->ic_opmode != IEEE80211_M_STA) {
-				/* Sync rates for associated stations and 
+				/* Sync rates for associated stations and
 				 * neighbors. */
-				ieee80211_iterate_nodes(&ic->ic_sta, 
+				ieee80211_iterate_nodes(&ic->ic_sta,
 						ath_rate_cb, NULL);
 			}
-			ath_rate_newassoc(ic->ic_dev->priv, 
+			ath_rate_newassoc(ic->ic_dev->priv,
 					ATH_NODE(vap->iv_bss), 1);
 		}
 }
@@ -770,7 +779,7 @@ ath_rate_newstate(struct ieee80211vap *vap, enum ieee80211_state newstate)
 static void
 ath_timer_function(unsigned long data)
 {
-	struct minstrel_softc *ssc = (struct minstrel_softc *) data;
+	struct minstrel_softc *ssc = (struct minstrel_softc *)data;
 	struct ath_softc *sc = ssc->sc;
 	struct ieee80211com *ic;
 	struct net_device *dev = ssc->sc_dev;
@@ -798,7 +807,7 @@ ath_timer_function(unsigned long data)
 				ath_rate_statistics(sc, tmpvap->iv_bss);
 			}
 		} else
-			ieee80211_iterate_nodes(&ic->ic_sta, 
+			ieee80211_iterate_nodes(&ic->ic_sta,
 					ath_rate_statistics, sc);
 	}
 
@@ -809,14 +818,13 @@ ath_timer_function(unsigned long data)
 	if (timer == NULL)
 		DPRINTF(sc, "%s: timer is null - leave it\n", __func__);
 
-	timer->expires = jiffies + ((HZ * interval) / 1000);
-	add_timer(timer);
+	mod_timer(timer, jiffies + ((HZ * interval) / 1000));
 }
 
 static void
 ath_rate_statistics(void *arg, struct ieee80211_node *ni)
 {
-	struct ath_node *an = (struct ath_node *) ni;
+	struct ath_node *an = (struct ath_node *)ni;
 	struct ieee80211_rateset *rs = &ni->ni_rates;
 	struct minstrel_node *rn = ATH_NODE_MINSTREL(an);
 	unsigned int i;
@@ -833,12 +841,12 @@ ath_rate_statistics(void *arg, struct ieee80211_node *ni)
 			micro_secs = ONE_SECOND;
 
 		if (rn->rs_rateattempts[i] != 0) {
-			p = (rn->rs_ratesuccess[i] * 18000) / 
+			p = (rn->rs_ratesuccess[i] * 18000) /
 				rn->rs_rateattempts[i];
 			rn->rs_succ_hist[i] += rn->rs_ratesuccess[i];
 			rn->rs_att_hist[i]  += rn->rs_rateattempts[i];
 			rn->rs_thisprob[i] = p;
-			p = ((p * (100 - ath_ewma_level)) + 
+			p = ((p * (100 - ath_ewma_level)) +
 				(rn->rs_probability[i] * ath_ewma_level)) / 100;
 			rn->rs_probability[i] = p;
 			rn->rs_this_tp[i] = p * (ONE_SECOND / micro_secs);
@@ -853,9 +861,9 @@ ath_rate_statistics(void *arg, struct ieee80211_node *ni)
 
 		/* Sample less often below the 10% chance of success.
 		 * Sample less often above the 95% chance of success.
-		 * 'rn->rs_probability' has a scale of 0 (0%) to 18000 (100%), 
+		 * 'rn->rs_probability' has a scale of 0 (0%) to 18000 (100%),
 		 * which avoids rounding issues.*/
-		if ((rn->rs_probability[i] > 17100) || 
+		if ((rn->rs_probability[i] > 17100) ||
 				(rn->rs_probability[i] < 1800)) {
 			rn->retry_adjusted_count[i] = rn->retry_count[i] >> 1;
 			if (rn->retry_adjusted_count[i] > 2)
@@ -926,8 +934,7 @@ ath_rate_attach(struct ath_softc *sc)
 	osc->timer.function = ath_timer_function;
 	osc->timer.data = (unsigned long)osc;
 
-	osc->timer.expires = jiffies + HZ;
-	add_timer(&osc->timer);
+	mod_timer(&osc->timer, jiffies + HZ);
 
 	return &osc->arc;
 }
@@ -935,7 +942,7 @@ ath_rate_attach(struct ath_softc *sc)
 static void
 ath_rate_detach(struct ath_ratectrl *arc)
 {
- 	struct minstrel_softc *osc = (struct minstrel_softc *) arc;
+ 	struct minstrel_softc *osc = (struct minstrel_softc *)arc;
 		osc->close_timer_now = 1;
 		del_timer(&osc->timer);
 		kfree(osc);
@@ -950,7 +957,7 @@ ath_proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 	struct ieee80211_node *ni;
 	struct minstrel_node *odst;
 	struct ieee80211_node_table *nt =
-		(struct ieee80211_node_table *) &vap->iv_ic->ic_sta;
+		(struct ieee80211_node_table *)&vap->iv_ic->ic_sta;
 	unsigned int x = 0;
 	unsigned int this_tp, this_prob, this_eprob;
 	struct ath_softc *sc = vap->iv_ic->ic_dev->priv;;
@@ -960,8 +967,8 @@ ath_proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 		/* Assume each node needs 1500 bytes */
 		if ((buf + space) < (p + 1500)) {
 			if ((buf + space) > (p + 100)) {
-				p += sprintf(p, "out of room for node " 
-						MAC_FMT "\n\n", 
+				p += sprintf(p, "out of room for node "
+						MAC_FMT "\n\n",
 						MAC_ADDR(ni->ni_macaddr));
 				break;
 			}
@@ -974,7 +981,7 @@ ath_proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 		if (IEEE80211_ADDR_EQ(vap->iv_myaddr, ni->ni_macaddr))
 			continue;
 
-		p += sprintf(p, "rate data for node: " MAC_FMT "\n", 
+		p += sprintf(p, "rate data for node: " MAC_FMT "\n",
 				MAC_ADDR(ni->ni_macaddr));
 		p += sprintf(p, "  rate   throughput EWMA prob this prob this "
 				"succ/attempt  success attempts\n");
@@ -990,7 +997,7 @@ ath_proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 
 			p += sprintf(p, " %2u%s",
 				     odst->rates[x].rate / 2,
-				     (odst->rates[x].rate & 0x1) != 0 ? 
+				     (odst->rates[x].rate & 0x1) != 0 ?
 				      ".5" : "  ");
 
 			this_tp = ((odst->rs_this_tp[x] / 18000) * 96) >> 10;
@@ -1009,7 +1016,7 @@ ath_proc_read_nodes(struct ieee80211vap *vap, char *buf, int space)
 		p += sprintf(p, "\n");
 
 		p += sprintf(p, "Total packet count::    ideal %d      "
-				"lookaround %d\n\n", 
+				"lookaround %d\n\n",
 				odst->packet_count, odst->sample_count);
 	}
 	IEEE80211_NODE_TABLE_UNLOCK_IRQ(nt);
@@ -1024,13 +1031,12 @@ ath_proc_ratesample_open(struct inode *inode, struct file *file)
 	struct proc_dir_entry *dp = PDE(inode);
 	struct ieee80211vap *vap = dp->data;
 
-	if (!(file->private_data = kmalloc(sizeof(struct proc_ieee80211_priv),
-					   GFP_KERNEL)))
+	if (!(file->private_data = kzalloc(sizeof(struct proc_ieee80211_priv),
+			   GFP_KERNEL)))
 		return -ENOMEM;
 
 	/* Initially allocate both read and write buffers */
-	pv = (struct proc_ieee80211_priv *) file->private_data;
-	memset(pv, 0, sizeof(struct proc_ieee80211_priv));
+	pv = (struct proc_ieee80211_priv *)file->private_data;
 	pv->rbuf = vmalloc(MAX_PROC_IEEE80211_SIZE);
 	if (!pv->rbuf) {
 		kfree(pv);
@@ -1094,14 +1100,15 @@ MODULE_LICENSE("Dual BSD/GPL");
 
 static int __init ath_rate_minstrel_init(void)
 {
-	printk(KERN_INFO "%s: Minstrel automatic rate control "
-	       "algorithm %s\n", dev_info, version);
+/* Debugging output - disabled as noisy. */
+#if 0
 	printk(KERN_INFO "%s: look around rate set to %d%%\n",
 	       dev_info, ath_lookaround_rate);
 	printk(KERN_INFO "%s: EWMA rolloff level set to %d%%\n",
 	       dev_info, ath_ewma_level);
-	printk(KERN_INFO "%s: max segment size in the mrr set "
+	printk(KERN_INFO "%s: max segment size in the MRR set "
 	       "to %d us\n", dev_info, ath_segment_size);
+#endif
 	return ieee80211_rate_register(&ath_rate_ops);
 }
 module_init(ath_rate_minstrel_init);
@@ -1109,7 +1116,6 @@ module_init(ath_rate_minstrel_init);
 static void __exit ath_rate_minstrel_exit(void)
 {
 	ieee80211_rate_unregister(&ath_rate_ops);
-	printk(KERN_INFO "%s: unloaded\n", dev_info);
 }
 module_exit(ath_rate_minstrel_exit);
 
