@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: onoe.c 2814 2007-10-31 05:39:41Z proski $
+ * $Id: onoe.c 3788 2008-07-17 05:11:56Z proski $
  */
 
 /*
@@ -160,7 +160,7 @@ ath_rate_tx_complete(struct ath_softc *sc,
 		on->on_tx_err++;
 	on->on_tx_retr += ts->ts_shortretry
 			+ ts->ts_longretry;
-	if (jiffies >= on->on_nextcheck) {
+	if (time_after_eq(jiffies, on->on_nextcheck)) {
 		ath_rate_ctl(sc, &an->an_node);
 		/* XXX halve rate for station mode */
 		on->on_nextcheck = jiffies + (ath_rateinterval * HZ) / 1000;
@@ -184,8 +184,8 @@ ath_rate_update(struct ath_softc *sc, struct ieee80211_node *ni, int rate)
 
 	KASSERT(rt != NULL, ("no rate table, mode %u", sc->sc_curmode));
 
-	DPRINTF(sc, "%s: set xmit rate for %s to %dM\n",
-		__func__, ether_sprintf(ni->ni_macaddr),
+	DPRINTF(sc, "%s: set xmit rate for " MAC_FMT " to %dM\n",
+		__func__, MAC_ADDR(ni->ni_macaddr),
 		ni->ni_rates.rs_nrates > 0 ?
 			(ni->ni_rates.rs_rates[rate] & IEEE80211_RATE_VAL) / 2 : 0);
 
@@ -362,8 +362,8 @@ ath_rate_ctl(void *arg, struct ieee80211_node *ni)
 	    on->on_tx_retr < (on->on_tx_ok * ath_rate_raise) / 100)
 		dir = 1;
 
-	DPRINTF(sc, "%s: ok %d err %d retr %d upper %d dir %d\n",
-		ether_sprintf(ni->ni_macaddr),
+	DPRINTF(sc, MAC_FMT ": ok %d err %d retr %d upper %d dir %d\n",
+		MAC_ADDR(ni->ni_macaddr),
 		on->on_tx_ok, on->on_tx_err, on->on_tx_retr,
 		on->on_tx_upper, dir);
 
@@ -423,7 +423,7 @@ ath_rate_attach(struct ath_softc *sc)
 static void
 ath_rate_detach(struct ath_ratectrl *arc)
 {
-	struct onoe_softc *osc = (struct onoe_softc *) arc;
+	struct onoe_softc *osc = (struct onoe_softc *)arc;
 
 	kfree(osc);
 	_MOD_DEC_USE(THIS_MODULE);
@@ -467,7 +467,7 @@ static ctl_table ath_rate_static_sysctls[] = {
 };
 static ctl_table ath_rate_table[] = {
 	{ .ctl_name	= CTL_AUTO,
-	  .procname	= "rate",
+	  .procname	= "rate_onoe",
 	  .mode		= 0555,
 	  .child	= ath_rate_static_sysctls
 	}, { 0 }
@@ -502,8 +502,10 @@ static struct ieee80211_rate_ops ath_rate_ops = {
 };
 
 #include "release.h"
+#if 0
 static char *version = "1.0 (" RELEASE_VERSION ")";
 static char *dev_info = "ath_rate_onoe";
+#endif
 
 MODULE_AUTHOR("Errno Consulting, Sam Leffler");
 MODULE_DESCRIPTION("Atsushi Onoe's rate control algorithm for Atheros devices");
@@ -517,10 +519,7 @@ MODULE_LICENSE("Dual BSD/GPL");
 static int __init
 init_ath_rate_onoe(void)
 {
-	int ret;
-	printk(KERN_INFO "%s: %s\n", dev_info, version);
-
-	ret = ieee80211_rate_register(&ath_rate_ops);
+	int ret = ieee80211_rate_register(&ath_rate_ops);
 	if (ret)
 		return ret;
 
@@ -535,7 +534,5 @@ exit_ath_rate_onoe(void)
 	if (ath_sysctl_header != NULL)
 		unregister_sysctl_table(ath_sysctl_header);
 	ieee80211_rate_unregister(&ath_rate_ops);
-
-	printk(KERN_INFO "%s: unloaded\n", dev_info);
 }
 module_exit(exit_ath_rate_onoe);
