@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_proto.h 3162 2008-01-16 20:01:39Z mtaylor $
+ * $Id: ieee80211_proto.h 3713 2008-06-08 01:43:53Z mentor $
  */
 #ifndef _NET80211_IEEE80211_PROTO_H_
 #define _NET80211_IEEE80211_PROTO_H_
@@ -55,6 +55,7 @@ extern const char *ieee80211_ctl_subtype_name[];
 extern const char *ieee80211_state_name[IEEE80211_S_MAX];
 extern const char *ieee80211_wme_acnames[];
 extern const char *ieee80211_phymode_name[];
+extern const char *ieee80211_opmode_name[];
 
 void ieee80211_proto_attach(struct ieee80211com *);
 void ieee80211_proto_detach(struct ieee80211com *);
@@ -63,14 +64,15 @@ void ieee80211_proto_vdetach(struct ieee80211vap *);
 
 struct ieee80211_node;
 struct ieee80211_channel *ieee80211_doth_findchan(struct ieee80211vap *, u_int8_t);
-int ieee80211_input(struct ieee80211_node *, struct sk_buff *, int, u_int64_t);
+int ieee80211_input(struct ieee80211vap *, struct ieee80211_node *, struct sk_buff *, int, u_int64_t);
 int ieee80211_input_all(struct ieee80211com *, struct sk_buff *, int, u_int64_t);
 int ieee80211_setup_rates(struct ieee80211_node *, const u_int8_t *,
 	const u_int8_t *, int);
 void ieee80211_saveie(u_int8_t **, const u_int8_t *);
 void ieee80211_saveath(struct ieee80211_node *, u_int8_t *);
-void ieee80211_recv_mgmt(struct ieee80211_node *, struct sk_buff *,
-	int, int, u_int64_t);
+int ieee80211_recv_mgmt(struct ieee80211vap *, struct ieee80211_node *,
+	struct sk_buff *, int, int, u_int64_t);
+void ieee80211_sta_pwrsave(struct ieee80211vap *, int);
 int ieee80211_hardstart(struct sk_buff *, struct net_device *);
 void ieee80211_parent_queue_xmit(struct sk_buff *);
 int ieee80211_send_nulldata(struct ieee80211_node *);
@@ -89,6 +91,7 @@ int ieee80211_iserp_rateset(struct ieee80211com *, struct ieee80211_rateset *);
 void ieee80211_set11gbasicrates(struct ieee80211_rateset *, enum ieee80211_phymode);
 enum ieee80211_phymode ieee80211_get11gbasicrates(struct ieee80211_rateset *);
 void ieee80211_send_pspoll(struct ieee80211_node *);
+void ieee80211_check_mic(struct ieee80211_node *, struct sk_buff *);
 
 /*
  * Return the size of the 802.11 header for a management or data frame.
@@ -249,7 +252,8 @@ void ieee80211_dturbo_switch(struct ieee80211com *, int);
 int ieee80211_new_state(struct ieee80211vap *, enum ieee80211_state, int);
 void ieee80211_print_essid(const u_int8_t *, int);
 #ifdef IEEE80211_DEBUG
-void ieee80211_dump_pkt(struct ieee80211com *, const u_int8_t *, int, int, int);
+void ieee80211_dump_pkt(struct ieee80211com *, const u_int8_t *,
+		int, int, int, int);
 #else
 #define ieee80211_dump_pkt(...)
 #endif
@@ -278,7 +282,7 @@ struct ieee80211_beacon_offsets {
 struct sk_buff *ieee80211_beacon_alloc(struct ieee80211_node *,
 	struct ieee80211_beacon_offsets *);
 int ieee80211_beacon_update(struct ieee80211_node *,
-	struct ieee80211_beacon_offsets *, struct sk_buff *, int);
+	struct ieee80211_beacon_offsets *, struct sk_buff *, int, int*);
 
 /* XXX exposed due to of beacon code botch */
 u_int8_t *ieee80211_add_rates(u_int8_t *, const struct ieee80211_rateset *);
