@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_var.h 3367 2008-03-01 07:59:04Z mentor $
+ * $Id: ieee80211_var.h 3808 2008-07-18 21:50:53Z proski $
  */
 
 #ifndef _NET80211_IEEE80211_VAR_H_
@@ -87,7 +87,7 @@
 	 IEEE80211_BMISSTHRESH_BMIN : (_bmt))
 
 #define	IEEE80211_BGSCAN_INTVAL_MIN	15	/* min bg scan intvl (secs) */
-#define	IEEE80211_BGSCAN_INTVAL_DEFAULT	(5*60)	/* default bg scan intvl */
+#define	IEEE80211_BGSCAN_INTVAL_DEFAULT	(5 * 60)	/* default BG scan int. */
 
 #define	IEEE80211_BGSCAN_IDLE_MIN	100	/* min idle time (ms) */
 #define	IEEE80211_BGSCAN_IDLE_DEFAULT	250	/* default idle time (ms) */
@@ -121,6 +121,16 @@
 	  "MadWifi" : \
 	  DEV_NAME(_v->iv_ic->ic_dev))
 
+/*
+ * TU conversions:
+ *
+ * TU is a 32bit value and defined by IEEE802.11 (page 6) as "A measurement of
+ * time equal to 1024 usec (microseconds)".
+ *
+ * TSF is a 64bit value in usec, therefore if we right shift it by 10 bit this
+ * will directly convert it to TU. The rest is truncated to fit into 32 bit.
+ */
+#define	IEEE80211_TSF_TO_TU(_tsf)	((u_int32_t)((_tsf) >> 10))
 #define	IEEE80211_MS_TO_TU(x)	(((x) * 1000) / 1024)
 #define	IEEE80211_TU_TO_MS(x)	(((x) * 1024) / 1000)
 #define	IEEE80211_TU_TO_JIFFIES(x) ((IEEE80211_TU_TO_MS(x) * HZ) / 1000)
@@ -219,7 +229,7 @@ struct ieee80211vap {
 	u_int iv_scanvalid;				/* scan cache valid threshold */
 	struct ieee80211_roam iv_roam;			/* sta-mode roaming state */
 
-	u_int32_t iv_csa_jiffies;			/* last csa recv jiffies */
+	unsigned long iv_csa_jiffies;			/* last csa recv jiffies */
 	u_int8_t iv_csa_count;				/* last csa count */
 	struct ieee80211_channel *iv_csa_chan;		/* last csa channel */
 	u_int8_t iv_csa_mode;				/* last csa mode */
@@ -229,7 +239,7 @@ struct ieee80211vap {
 	u_int16_t iv_sta_assoc;				/* stations associated */
 	u_int16_t iv_ps_sta;				/* stations in power save */
 	u_int16_t iv_ps_pending;			/* PS STAs w/ pending frames */
-	u_int8_t *iv_tim_bitmap;			/* power-save stations w/ data*/
+	u_int8_t *iv_tim_bitmap;			/* power-save stations w/ data */
 	u_int16_t iv_tim_len;				/* ic_tim_bitmap size (bytes) */
 	u_int8_t iv_dtim_period;			/* DTIM period */
 	u_int8_t iv_dtim_count;				/* DTIM count from last bcn */
@@ -283,7 +293,7 @@ struct ieee80211vap {
 	u_int32_t app_filter;				/* filters which management frames are forwarded to app */
 };
 
-/* Debug functions need the defintion of struct ieee80211vap because iv_debug 
+/* Debug functions need the definition of struct ieee80211vap because iv_debug
  * is used at runtime to determine if we should log an event
  */
 #include <net80211/ieee80211_debug.h>
@@ -294,7 +304,8 @@ struct ieee80211com {
 	struct net_device *ic_dev;		/* associated device */
 	ieee80211com_lock_t ic_comlock;		/* state update lock */
 	ieee80211com_lock_t ic_vapslock; 	/* vap state machine lock */
-	TAILQ_HEAD(, ieee80211vap) ic_vaps;	/* list of vap instances */
+	TAILQ_HEAD(ieee80211vap_headtype,
+			ieee80211vap) ic_vaps;	/* list of vap instances */
 	enum ieee80211_phytype ic_phytype;	/* XXX wrong for multi-mode */
 	enum ieee80211_opmode ic_opmode;	/* operation mode */
 	struct ifmedia ic_media;		/* interface media config */
@@ -306,7 +317,7 @@ struct ieee80211com {
 	u_int32_t ic_caps;			/* capabilities */
 	u_int8_t ic_ath_cap;			/* Atheros adv. capabilities */
 	u_int8_t ic_promisc;			/* VAPs needing promisc mode */
-	u_int8_t ic_allmulti;			/* VAPs needing all multicast*/
+	u_int8_t ic_allmulti;			/* VAPs needing all multicast */
 	u_int8_t ic_nopened;			/* VAPs been opened */
 	struct ieee80211_rateset ic_sup_rates[IEEE80211_MODE_MAX];
 	struct ieee80211_rateset ic_sup_xr_rates;
@@ -365,13 +376,13 @@ struct ieee80211com {
 	 *    association of the STA and disassociation of x other STAs (out of
 	 *    y associated STAs in total), the number of common channel
 	 *    increases by z, then such an action is performed if
-	 *    1000*x/y < z*ic_sc_slcg
+	 *    1000 * x/y < z * ic_sc_slcg
 	 * ic_sc_sldg is the permil of Stations Lost per rssi Db Gained, the
 	 *    parameter used by SC_LOOSE algorithm. If due to the switch,
 	 *    the maximum RSSI of received packets on the current channel would
 	 *    decrease by z decibels and x stations from the set of y stations
 	 *    would be lost, then such a switch will be performed if
-	 *    1000*x/y < z*ic_sc_sldg
+	 *    1000 * x/y < z * ic_sc_sldg
 	 * ic_sc_ie is the Supported Channels IE that is about to be sent along
 	 *    with (re)assoc requests (STA mode)
 	 */
@@ -413,7 +424,7 @@ struct ieee80211com {
 	int ic_country_outdoor;
 	struct ieee80211_ie_country ic_country_ie; /* country info element */
 
-	/* current channel max power, used to compute Power Constraint IE.
+	/* Current channel max. power, used to compute Power Constraint IE.
 	 *
 	 * NB: local power constraint depends on the channel, but assuming it must
 	 *     be detected dynamically, we cannot maintain a table (i.e., will not
@@ -435,7 +446,7 @@ struct ieee80211com {
 
 	/* Send/recv 802.11 management frame */
 	int (*ic_send_mgmt)(struct ieee80211_node *, int, int);
-	void (*ic_recv_mgmt)(struct ieee80211vap *, struct ieee80211_node *,
+	int (*ic_recv_mgmt)(struct ieee80211vap *, struct ieee80211_node *,
 		struct sk_buff *, int, int, u_int64_t);
 
 	/* Send management frame to driver (like hardstart) */
@@ -453,18 +464,9 @@ struct ieee80211com {
 
 	/* Node state management */
 	int32_t (*ic_node_count)(struct ieee80211com *);
-#ifdef IEEE80211_DEBUG_REFCNT
-	struct ieee80211_node *(*ic_node_alloc_debug)(struct ieee80211vap *, 
-			const char* func, int line);
-	void (*ic_node_cleanup_debug)(struct ieee80211_node *, 
-			const char* func, int line);
-	void (*ic_node_free_debug)(struct ieee80211_node *, 
-			const char* func, int line);
-#else
 	struct ieee80211_node *(*ic_node_alloc)(struct ieee80211vap *);
 	void (*ic_node_cleanup)(struct ieee80211_node *);
 	void (*ic_node_free)(struct ieee80211_node *);
-#endif
 
 	u_int8_t (*ic_node_getrssi)(const struct ieee80211_node *);
 	u_int8_t (*ic_node_move_data)(const struct ieee80211_node *);
@@ -518,6 +520,7 @@ struct ieee80211com {
 	unsigned int (*ic_write_register)(struct ieee80211com *, unsigned int, unsigned int);
 	unsigned int (*ic_read_register)(struct ieee80211com *, unsigned int, unsigned int*);
 #endif /* #ifdef ATH_REVERSE_ENGINEERING */
+	int (*ic_debug_ath_iwpriv)(struct ieee80211com *, unsigned int param, unsigned int value);
 };
 
 #define MAX_PROC_IEEE80211_SIZE 16383
@@ -583,7 +586,7 @@ MALLOC_DECLARE(M_80211_VAP);
 #define IEEE80211_F_NOBRIDGE    0x10000000		/* CONF: disable internal bridge */
 
 #define	IEEE80211_F_WMEUPDATE	0x20000000		/* STATUS: update beacon wme */
-#define IEEE80211_F_DOTH	0x40000000		/* enable 11.h */
+#define IEEE80211_F_DOTH	0x40000000		/* CONF: enable 11.h (dot11SpectrumManagementRequired) */
 #define IEEE80211_F_CHANSWITCH	0x80000000		/* force chanswitch */
 
 /* ic_flags_ext */
