@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: wlanconfig.c 2988 2007-11-28 05:36:29Z proski $
+ * $Id: wlanconfig.c 3710 2008-06-06 20:32:29Z mentor $
  */
 
 /*
@@ -146,7 +146,7 @@ main(int argc, char *argv[])
 				strncpy(ifr.ifr_name, argv[4], IFNAMSIZ);
 				argc--;
 				argv++;
-			} else if (strcmp(argv[3], "nounit" ) == 0) {
+			} else if (strcmp(argv[3], "nounit") == 0) {
 				bnounit = 1;
 			} else {
 				int flag = getflag(argv[3]);
@@ -173,7 +173,7 @@ main(int argc, char *argv[])
 		free(if_base);
 		if_base = NULL;
 
-		ifr.ifr_data = (void *) &cp;
+		ifr.ifr_data = (void *)&cp;
 		vap_create(&ifr);
 		printf("%s\n", ifr.ifr_name);
 	} else if (streq(cmd, "destroy")) {
@@ -284,7 +284,7 @@ static void
 usage(void)
 {
 	fprintf(stderr, "usage: wlanconfig athX create [nounit] wlandev wifiY\n");
-	fprintf(stderr, "            wlanmode [sta|adhoc|ap|monitor|wds|ahdemo] [bssid | -bssid] [nosbeacon]\n");
+	fprintf(stderr, "            wlanmode [sta|adhoc|ap|monitor|wds|ahdemo] [uniquebssid]\n");
 	fprintf(stderr, "usage: wlanconfig athX destroy\n");
 	fprintf(stderr, "usage: wlanconfig athX list [active|ap|caps|chan|freq|keys|scan|sta|wme]\n");
 	exit(-1);
@@ -318,10 +318,16 @@ getflag(const char  *s)
 	int flag = 0;
 
 	cp = (s[0] == '-' ? s + 1 : s);
-	if (strcmp(cp, "bssid") == 0)
+	if (strcmp(cp, "bssid") == 0) {
+		printf("WARNING: the -bssid and bssid flags are deprecated.\n");
 		flag = IEEE80211_CLONE_BSSID;
-	if (strcmp(cp, "nosbeacon") == 0)
-		flag |= IEEE80211_NO_STABEACONS;
+	}
+	if (strcmp(cp, "uniquebssid") == 0)
+		flag = IEEE80211_CLONE_BSSID;
+	if (strcmp(cp, "nosbeacon") == 0) {
+		printf("WARNING: the nosbeacon flag has been deprecated.\n");
+		return 0; /* deprecated but skip */
+	}
 	if (flag == 0)
 		errx(1, "unknown create option %s", s);
 	return (s[0] == '-' ? -flag : flag);
@@ -428,12 +434,12 @@ getstamode(u_int8_t opmode)
 }
 
 static void
-printie(const char* tag, const uint8_t *ie, size_t ielen, int maxlen)
+printie(const char *tag, const uint8_t *ie, size_t ielen, int maxlen)
 {
 	printf("%s", tag);
 	if (verbose) {
-		maxlen -= strlen(tag)+2;
-		if (2*ielen > maxlen)
+		maxlen -= strlen(tag) + 2;
+		if (2 * ielen > maxlen)
 			maxlen--;
 		printf("<");
 		for (; ielen > 0; ie++, ielen--) {
@@ -559,7 +565,7 @@ ieee80211_ntoa(const uint8_t mac[IEEE80211_ADDR_LEN])
 static void
 list_stations(const char *ifname)
 {
-	uint8_t buf[24*1024];
+	uint8_t buf[24 * 1024];
 	struct iwreq iwr;
 	uint8_t *cp;
 	int s, len;
@@ -570,7 +576,7 @@ list_stations(const char *ifname)
 
 	(void) memset(&iwr, 0, sizeof(iwr));
 	(void) strncpy(iwr.ifr_name, ifname, sizeof(iwr.ifr_name));
-	iwr.u.data.pointer = (void *) buf;
+	iwr.u.data.pointer = (void *)buf;
 	iwr.u.data.length = sizeof(buf);
 	if (ioctl(s, IEEE80211_IOCTL_STA_INFO, &iwr) < 0)
 		errx(1, "unable to get station information");
@@ -601,7 +607,7 @@ list_stations(const char *ifname)
 		struct ieee80211req_sta_info *si;
 		uint8_t *vp;
 
-		si = (struct ieee80211req_sta_info *) cp;
+		si = (struct ieee80211req_sta_info *)cp;
 		vp = (u_int8_t *)(si+1);
 		printf("%s %4u %4d %3dM %4d %4d %5d %6d %7d %6d %7d %-4.4s %-5.5s %3x %8x %8s",
 			ieee80211_ntoa(si->isi_macaddr),
@@ -677,8 +683,8 @@ list_scan(const char *ifname)
 		struct ieee80211req_scan_result *sr;
 		uint8_t *vp;
 
-		sr = (struct ieee80211req_scan_result *) cp;
-		vp = (u_int8_t *)(sr+1);
+		sr = (struct ieee80211req_scan_result *)cp;
+		vp = (u_int8_t *)(sr + 1);
 		printf("%-14.*s  %s  %3d  %3dM %2d:%-2d  %3d %-4.4s",
 			copy_essid(ssid, sizeof(ssid), vp, sr->isr_ssid_len),
 			ssid,
@@ -963,6 +969,7 @@ do80211priv(struct iwreq *iwr, const char *ifname, int op, void *data, size_t le
 			IOCTL_ERR(IEEE80211_IOCTL_RADAR),
 			IOCTL_ERR(IEEE80211_IOCTL_SETKEY),
 			IOCTL_ERR(IEEE80211_IOCTL_DELKEY),
+			IOCTL_ERR(IEEE80211_IOCTL_HALMAP),
 			IOCTL_ERR(IEEE80211_IOCTL_ADDMAC),
 			IOCTL_ERR(IEEE80211_IOCTL_DELMAC),
 			IOCTL_ERR(IEEE80211_IOCTL_WDSADDMAC),
